# coding=utf-8
"""
"""
__author__ = 'Alisue <lambdalisue@hashnote.net>'
from roughpages.conf import settings
from roughpages.compat import import_module


def get_backend(backend_class=None):
    """
    Get backend instance

    If no `backend_class` is specified, the backend class is determined from
    the value of `settings.ROUGHPAGES_BACKEND`.
    `backend_class` can be a class object or dots separated python import path

    Returns:
        backend instance
    """
    cache_name = '_backend_instance'
    if not hasattr(get_backend, cache_name):
        backend_class = backend_class or settings.ROUGHPAGES_BACKEND
        if isinstance(backend_class, basestring):
            module_path, class_name = backend_class.rsplit(".", 1)
            module = import_module(module_path)
            backend_class = getattr(module, class_name)
        setattr(get_backend, cache_name, backend_class())
    return getattr(get_backend, cache_name)


from roughpages.backends.decorators import prepare_filename_decorator
from roughpages.backends.base import TemplateFilenameBackendBase
from roughpages.backends.plain import PlainTemplateFilenameBackend
from roughpages.backends.auth import AuthTemplateFilenameBackend



# coding=utf-8
"""
"""
__author__ = 'Alisue <lambdalisue@hashnote.net>'
from roughpages.backends.base import TemplateFilenameBackendBase
from roughpages.backends.decorators import prepare_filename_decorator


class PlainTemplateFilenameBackend(TemplateFilenameBackendBase):
    """
    A TemplateFilenameBackend which simply return filename single list
    """

    @prepare_filename_decorator
    def prepare_filenames(self, normalized_url, request):
        """
        Prepare template filename list

        Args:
            normalized_url (str): A normalized url
            request (instance): An instance of HttpRequest

        Returns:
            list

        Examples:
            >>> from mock import MagicMock
            >>> request = MagicMock()
            >>> backend = PlainTemplateFilenameBackend()
            >>> filenames = backend.prepare_filenames('foo/bar/hogehoge',
            ...                                       request)
            >>> assert filenames == [
            ...     'foo/bar/hogehoge.html'
            ... ]
            >>> filenames = backend.prepare_filenames('',
            ...                                       request)
            >>> assert filenames == [
            ...     'index.html'
            ... ]
        """
        return [normalized_url]



# coding=utf-8
"""
"""
__author__ = 'Alisue <lambdalisue@hashnote.net>'
import os


def url_to_filename(url):
    """
    Safely translate url to relative filename

    Args:
        url (str): A target url string

    Returns:
        str
    """
    # remove leading/trailing slash
    if url.startswith('/'):
        url = url[1:]
    if url.endswith('/'):
        url = url[:-1]
    # remove pardir symbols to prevent unwilling filesystem access
    url = remove_pardir_symbols(url)
    return url


def remove_pardir_symbols(path, sep=os.sep, pardir=os.pardir):
    """
    Remove relative path symobls such as '..'

    Args:
        path (str): A target path string
        sep (str): A strint to refer path delimiter (Default: `os.sep`)
        pardir (str): A string to refer parent directory (Default: `os.pardir`)

    Returns:
        str
    """
    bits = path.split(sep)
    bits = (x for x in bits if x != pardir)
    return sep.join(bits)


if __name__ == '__main__':
    import doctest
    doctest.testmod()



# coding=utf-8
"""
"""
__author__ = 'Alisue <lambdalisue@hashnote.net>'
from roughpages.backends.base import TemplateFilenameBackendBase
from roughpages.backends.decorators import prepare_filename_decorator


class AuthTemplateFilenameBackend(TemplateFilenameBackendBase):
    """
    A TemplateFilenameBackend which return filename with authenticated state
    suffix.
    """

    @prepare_filename_decorator
    def prepare_filenames(self, normalized_url, request):
        """
        Prepare template filename list based on the user authenticated state

        If user is authenticated user, it use '_authenticated' as a suffix.
        Otherwise it use '_anonymous' as a suffix to produce the template
        filename list. The list include original filename at the end of the
        list.

        Args:
            normalized_url (str): A normalized url
            request (instance): An instance of HttpRequest

        Returns:
            list

        Examples:
            >>> from mock import MagicMock
            >>> request = MagicMock()
            >>> backend = AuthTemplateFilenameBackend()
            >>> request.user.is_authenticated.return_value = True
            >>> filenames = backend.prepare_filenames('foo/bar/hogehoge',
            ...                                       request)
            >>> assert filenames == [
            ...     'foo/bar/hogehoge_authenticated.html',
            ...     'foo/bar/hogehoge.html'
            ... ]
            >>> request.user.is_authenticated.return_value = False
            >>> filenames = backend.prepare_filenames('foo/bar/hogehoge',
            ...                                       request)
            >>> assert filenames == [
            ...     'foo/bar/hogehoge_anonymous.html',
            ...     'foo/bar/hogehoge.html'
            ... ]
            >>> request.user.is_authenticated.return_value = True
            >>> filenames = backend.prepare_filenames('',
            ...                                       request)
            >>> assert filenames == [
            ...     'index_authenticated.html',
            ...     'index.html'
            ... ]
            >>> request.user.is_authenticated.return_value = False
            >>> filenames = backend.prepare_filenames('',
            ...                                       request)
            >>> assert filenames == [
            ...     'index_anonymous.html',
            ...     'index.html'
            ... ]
        """
        filenames = [normalized_url]
        if request.user.is_authenticated():
            filenames.insert(0, normalized_url + "_authenticated")
        else:
            filenames.insert(0, normalized_url + "_anonymous")
        return filenames



# Python 2.7 has an importlib with import_module; for older Pythons,
# Django's bundled copy provides it.
try:
    from importlib import import_module
except ImportError:
    from django.utils.importlib import import_module



try:
    # Python 3 have mock in unittest
    from unittest.mock import MagicMock, patch, DEFAULT
except ImportError:
    from mock import MagicMock, patch, DEFAULT

try:
    from django.test.utils import override_settings
except ImportError:
    from override_settings import override_settings

try:
    from unittest import skipIf
except ImportError:
    def skipIf(condition, message):
        def decorator(f):
            return None if condition else f
        return decorator



# coding=utf-8
"""
"""
__author__ = 'Alisue <lambdalisue@hashnote.net>'
from functools import wraps
from roughpages.conf import settings


def prepare_filename_decorator(fn):
    """
    A decorator of `prepare_filename` method

    1. It automatically assign `settings.ROUGHPAGES_INDEX_FILENAME` if the
       `normalized_url` is ''.
    2. It automatically assign file extensions to the output list.
    """
    @wraps(fn)
    def inner(self, normalized_url, request):
        ext = settings.ROUGHPAGES_TEMPLATE_FILE_EXT
        if not normalized_url:
            normalized_url = settings.ROUGHPAGES_INDEX_FILENAME
        filenames = fn(self, normalized_url, request)
        filenames = [x + ext for x in filenames if x]
        return filenames
    return inner



# coding=utf-8
"""
"""
__author__ = 'Alisue <lambdalisue@hashnote.net>'


class TemplateFilenameBackendBase(object):
    """
    A base class of TemplateFilenameBackend
    """
    def prepare_filenames(self, normalized_url, request):
        """
        Prepare template filename list

        Args:
            normalized_url (str): A normalized url
            request (instance): An instance of HttpRequest

        Returns:
            list

        Raises:
            NotImplementedError
        """
        raise NotImplementedError(
            "Subclass of TemplateFilenameBackendBase need to "
            "override 'prepare_filename' method.")




# coding=utf-8
"""
"""
__author__ = 'Alisue <lambdalisue@hashnote.net>'
from django.test import TestCase
from roughpages.utils import url_to_filename, remove_pardir_symbols
from roughpages.tests.compat import patch


class RoughpagesUtilsTestCase(TestCase):
    def test_url_to_filename_remove_leading_slash(self):
        url = "/aaa/bbb/ccc"
        expected = "aaa/bbb/ccc"
        self.assertEqual(url_to_filename(url),
                         expected)

    def test_url_to_filename_remove_trailing_slash(self):
        url = "aaa/bbb/ccc/"
        expected = "aaa/bbb/ccc"
        self.assertEqual(url_to_filename(url),
                         expected)

    def test_url_to_filename_remove_call_remove_pardir_symbols(self):
        with patch('roughpages.utils.remove_pardir_symbols') as p:
            url_to_filename("")
            self.assertTrue(p.called)

    def test_remove_pardir_symbols_remove_pardir_symobls(self):
        path = "/../A/../B/../C/../"
        expected = "/A/B/C/"
        self.assertEqual(remove_pardir_symbols(path),
                         expected)



# coding=utf-8
"""
"""
__author__ = 'Alisue <lambdalisue@hashnote.net>'
from django.test import TestCase
from django.http import Http404
from roughpages.middleware import RoughpageFallbackMiddleware
from roughpages.tests.compat import MagicMock, patch, override_settings


@override_settings(DEBUG=True)
class RoughpagesRoughpageFallbackMiddlewareTestCase(TestCase):
    def setUp(self):
        self.middleware = RoughpageFallbackMiddleware()
        self.request = MagicMock()
        self.request.session = {}
        self.request.path_info = '/foo/bar/hoge/'
        self.response = MagicMock()

    def test_middleware_with_200_response(self):
        self.response.status_code = 200
        with patch('roughpages.middleware.roughpage') as p:
            r = self.middleware.process_response(self.request,
                                                 self.response)
            # roughpage view should not be called
            self.assertFalse(p.called)
            # original response should be returend
            self.assertEqual(r, self.response)

    def test_middleware_with_404_response(self):
        self.response.status_code = 404
        with patch('roughpages.middleware.roughpage') as p:
            r = self.middleware.process_response(self.request,
                                                 self.response)
            # roughpage view should be called
            self.assertTrue(p.called)
            # roughpage view should be called with request and path_info
            p.assert_called_with(self.request, self.request.path_info)
            # roughpage return should be returned
            self.assertEqual(r, p())

    def test_middleware_with_404_response_with_404(self):
        self.response.status_code = 404
        with patch('roughpages.middleware.roughpage') as p:
            p.side_effect = Http404
            r = self.middleware.process_response(self.request,
                                                 self.response)
            # roughpage view should be called
            self.assertTrue(p.called)
            # roughpage view should be called with request and path_info
            p.assert_called_with(self.request, self.request.path_info)
            # original response should be returend
            self.assertEqual(r, self.response)

    def test_middleware_with_404_response_with_exception(self):
        self.response.status_code = 404
        with patch('roughpages.middleware.roughpage') as p:
            p.side_effect = Exception
            self.assertRaises(Exception,
                              self.middleware.process_response,
                              self.request, self.response)
            # roughpage view should be called
            self.assertTrue(p.called)
            # roughpage view should be called with request and path_info
            p.assert_called_with(self.request, self.request.path_info)

    @override_settings(DEBUG=False)
    def test_middleware_with_404_response_with_exception_NO_DEBUG(self):
        self.response.status_code = 404
        with patch('roughpages.middleware.roughpage') as p:
            p.side_effect = Exception
            r = self.middleware.process_response(self.request,
                                                 self.response)
            # roughpage view should be called
            self.assertTrue(p.called)
            # roughpage view should be called with request and path_info
            p.assert_called_with(self.request, self.request.path_info)
            # original response should be returend
            self.assertEqual(r, self.response)



# coding=utf-8
"""
"""
__author__ = 'Alisue <lambdalisue@hashnote.net>'
from django.test import TestCase
from django.http import Http404
from django.template import TemplateDoesNotExist
from roughpages.tests.compat import MagicMock, override_settings
from roughpages.tests.compat import patch, DEFAULT
from roughpages.views import render_roughpage, roughpage


@override_settings(
    APPEND_SLASH=True,
    ROUGHPAGES_BACKEND='roughpages.backends.PlainTemplateFilenameBackend',
    ROUGHPAGES_TEMPLATE_DIR='roughpages',
    ROUGHPAGES_TEMPLATE_FILE_EXT='.html',
    ROUGHPAGES_RAISE_TEMPLATE_DOES_NOT_EXISTS=False,
)
class RoughpagesViewsTestCase(TestCase):
    def setUp(self):
        self.request = MagicMock()
        self.request.session = {}
        self.request.user.is_authenticated.return_value = False

    @patch.multiple('roughpages.views',
                    RequestContext=DEFAULT,
                    HttpResponse=DEFAULT)
    def test_render_roughpage(self, RequestContext, HttpResponse):
        t = MagicMock()
        r = render_roughpage(self.request, t)
        # RequestContext should be initialized with request
        RequestContext.assert_called_with(self.request)
        # t.render should be called with the context
        t.render.assert_called_with(RequestContext())
        # HttpResponse should be initialized with the output of
        # t.render(c)
        HttpResponse.assert_called_with(t.render(RequestContext()))
        # return should be response
        self.assertEqual(r, HttpResponse())

    @patch.multiple('roughpages.views',
                    loader=DEFAULT,
                    render_roughpage=DEFAULT)
    def test_roughpage(self, loader, render_roughpage):
        url = '/foo/bar/hoge/'
        r = roughpage(self.request, url)
        # loader.select_template should be called with follow
        # because backend is PlainTemplateFilenameBackend
        template_filenames = ['roughpages/foo/bar/hoge.html']
        loader.select_template.assert_called_with(template_filenames)
        # render_roughpage should be called with
        t = loader.select_template()
        render_roughpage.assert_called_with(self.request, t)
        # should return the return of render_roughpage
        self.assertEqual(r, render_roughpage())

    @patch.multiple('roughpages.views',
                    redirect=DEFAULT)
    def test_roughpage_redirect(self, redirect):
        url = '/foo/bar/hoge'
        roughpage(self.request, url)
        # redirect should be called
        redirect.assert_called_with(url + '/', permanent=True)

    @patch.multiple('roughpages.views',
                    loader=DEFAULT,
                    render_roughpage=DEFAULT)
    def test_roughpage_index(self, loader, render_roughpage):
        url = '/'
        r = roughpage(self.request, url)
        # loader.select_template should be called with follow
        # because backend is PlainTemplateFilenameBackend
        template_filenames = ['roughpages/index.html']
        loader.select_template.assert_called_with(template_filenames)
        # render_roughpage should be called with
        t = loader.select_template()
        render_roughpage.assert_called_with(self.request, t)
        # should return the return of render_roughpage
        self.assertEqual(r, render_roughpage())

    @patch.multiple('roughpages.views',
                    get_backend=DEFAULT,
                    loader=DEFAULT,
                    render_roughpage=DEFAULT)
    def test_roughpage_index_auth(self, get_backend, loader, render_roughpage):
        from roughpages.backends import AuthTemplateFilenameBackend
        get_backend.return_value = AuthTemplateFilenameBackend()
        url = '/'
        r = roughpage(self.request, url)
        # loader.select_template should be called with follow
        # because backend is PlainTemplateFilenameBackend
        template_filenames = [
            'roughpages/index_anonymous.html',
            'roughpages/index.html',
        ]
        loader.select_template.assert_called_with(template_filenames)
        # render_roughpage should be called with
        t = loader.select_template()
        render_roughpage.assert_called_with(self.request, t)
        # should return the return of render_roughpage
        self.assertEqual(r, render_roughpage())

    @patch.multiple('roughpages.views',
                    loader=DEFAULT,
                    render_roughpage=DEFAULT)
    def test_roughpage_no_template(self, loader, render_roughpage):
        loader.select_template.side_effect = TemplateDoesNotExist
        url = '/foo/bar/hoge/'
        self.assertRaises(Http404,
                          roughpage,
                          self.request, url)
        # loader.select_template should be called with follow
        # because backend is PlainTemplateFilenameBackend
        template_filenames = ['roughpages/foo/bar/hoge.html']
        loader.select_template.assert_called_with(template_filenames)

    @override_settings(
        ROUGHPAGES_RAISE_TEMPLATE_DOES_NOT_EXISTS=True)
    @patch.multiple('roughpages.views',
                    loader=DEFAULT,
                    render_roughpage=DEFAULT)
    def test_roughpage_no_template_raise(self, loader, render_roughpage):
        loader.select_template.side_effect = TemplateDoesNotExist
        url = '/foo/bar/hoge/'
        self.assertRaises(TemplateDoesNotExist,
                          roughpage,
                          self.request, url)
        # loader.select_template should be called with follow
        # because backend is PlainTemplateFilenameBackend
        template_filenames = ['roughpages/foo/bar/hoge.html']
        loader.select_template.assert_called_with(template_filenames)



# coding=utf-8
"""
Roughpage Middleware

Ref: https://github.com/django/django/blob/master/
     django/contrib/flatpages/middleware.py
"""
__author__ = 'Alisue <lambdalisue@hashnote.net>'
from django.http import Http404
from roughpages.conf import settings
from roughpages.views import roughpage


class RoughpageFallbackMiddleware(object):
    def process_response(self, request, response):
        if response.status_code != 404:
            # Non 404 response should not be treated with this middleware
            return response
        try:
            return roughpage(request, request.path_info)
        # Return the original response if any errors happened. Because this
        # is a middleware, we can't assume the errors will be caught elsewhere.
        except Http404:
            return response
        except Exception:
            if settings.DEBUG:
                raise
            return response



# coding=utf-8
"""
django-permission application configure
"""
__author__ = 'Alisue <lambdalisue@hashnote.net>'
__all__ = ('settings',)
from django.conf import settings
from appconf import AppConf


class RoughpageConf(AppConf):
    # An backend class path
    BACKEND = 'roughpages.backends.AuthTemplateFilenameBackend'

    # A special filename without extension indicate the root '/'
    INDEX_FILENAME = 'index'

    # A template filename located directory
    TEMPLATE_DIR = 'roughpages'

    # A template filename extension string
    TEMPLATE_FILE_EXT = '.html'

    # Raise TemplateDoesNotExist exception
    RAISE_TEMPLATE_DOES_NOT_EXISTS = False
